/* Copyright (c) 2017 Felix Kutzner (github.com/fkutzner)
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 
 Except as contained in this notice, the name(s) of the above copyright holders
 shall not be used in advertising or otherwise to promote the sale, use or
 other dealings in this Software without prior written authorization.
 
 */

#ifndef X_4311DBC6_D73E_4BCA_A592_823CDA996EB9_TESTUTILS_H
#define X_4311DBC6_D73E_4BCA_A592_823CDA996EB9_TESTUTILS_H

#include <candy/core/SolverTypes.h>
#include <unordered_set>
#include <unordered_map>
#include <vector>


namespace Candy {
    class CNFProblem;
}

namespace Candy {
    class Conjectures;
    class EquivalenceConjecture;
    
    /** \defgroup TestUtils */
    
    /**
     * \ingroup TestUtils
     *
     * Asserts that variables does not contain the variable forbidden.
     */
    void assertContainsVariable(const std::unordered_set<Var>& variables, const Var forbidden);
    
    /**
     * \ingroup TestUtils
     *
     * Asserts that variables contains the variable required.
     */
    void assertDoesNotContainVariable(const std::unordered_set<Var>& variables, const Var required);
    
    /**
     * \ingroup TestUtils
     *
     * Returns a clause containing the negated lits of the given clause.
     */
    Cl negatedLits(const Cl& clause);
    
    /**
     * \ingroup TestUtils
     *
     * Inserts the variables contained in the literals of lits in to the target set.
     */
    void insertVariables(const std::vector<Lit>& lits, std::unordered_set<Var>& target);
    
    /**
     * \ingroup TestUtils
     *
     * Returns true iff the given formula contains the given clause.
     */
    bool containsClause(const CNFProblem& formula, const Cl& clause);
    
    /** 
     * \ingroup TestUtils
     *
     * Gets the maximum value of |x - y| where x rsp. y are values of sample1 rsp. sample2 having
     * the same key.
     */
    double getMaxAbsDifference(const std::unordered_map<std::uint8_t, double>& sample1,
                               const std::unordered_map<std::uint8_t, double>& sample2);
}

#endif
